import { __awaiter } from 'tslib';
import { createInstance, LOCALSTORAGE } from 'localforage';
import { SubscriptionClient } from 'subscriptions-transport-ws';
import { ApolloClient } from 'apollo-client';
import { createHttpLink } from 'apollo-link-http';
import { WebSocketLink } from 'apollo-link-ws';
import { InMemoryCache } from 'apollo-cache-inmemory';
import { split } from 'apollo-link';
import { getMainDefinition } from 'apollo-utilities';
import { delay } from 'bluebird';
import { NgModule, APP_INITIALIZER, Optional, SkipSelf } from '@angular/core';

/**
 * @fileoverview added by tsickle
 * Generated from: lib/services/ec.service.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @record
 */
function ECLite() { }
if (false) {
    /** @type {?} */
    ECLite.prototype.fetch;
    /** @type {?} */
    ECLite.prototype.WebSocket;
}
class EdgeCentralConfig {
}
if (false) {
    /** @type {?} */
    EdgeCentralConfig.prototype.cloudCentralHost;
    /** @type {?} */
    EdgeCentralConfig.prototype.edgeCN;
    /** @type {?} */
    EdgeCentralConfig.prototype.ecvcHost;
    /** @type {?} */
    EdgeCentralConfig.prototype.inSecure;
}
/**
 * @record
 */
function EdgeCentralClientConfig() { }
if (false) {
    /** @type {?} */
    EdgeCentralClientConfig.prototype.apiUrl;
    /** @type {?} */
    EdgeCentralClientConfig.prototype.subscriptionUrl;
    /** @type {?} */
    EdgeCentralClientConfig.prototype.token;
}
class EdgeCentralClient {
    /* eslint-disable no-useless-constructor */
    /**
     * @param {?} config
     */
    constructor(config) {
        this.config = config;
        this.httpProtocol = 'https';
        this.wsProtocol = 'wss';
        this.store = createInstance({
            name: 'ec-client',
            driver: LOCALSTORAGE
        });
        if (config.inSecure) {
            // set protocol to insecure protocol
            this.httpProtocol = 'http';
            this.wsProtocol = 'ws';
        }
    }
    /**
     * @return {?}
     */
    isAuthenticated() {
        return __awaiter(this, void 0, void 0, function* () {
            // TODO: use more shopisticated detection mechanism
            yield delay(500);
            /* eslint-disable no-console */
            console.log('using chrome extensions?', this.isUsingChromeExtension());
            if (this.isUsingChromeExtension()) {
                return true;
            }
            /** @type {?} */
            const config = yield this.getClientConfig();
            if (!config.token) {
                return false;
            }
            return true;
        });
    }
    /**
     * return true when chrome inject script from ec-lite extenstion are presents
     * @return {?}
     */
    isUsingChromeExtension() {
        if (window.eclite) {
            return true;
        }
        return false;
    }
    /**
     * @return {?}
     */
    loadApolloClient() {
        return __awaiter(this, void 0, void 0, function* () {
            this.client = yield this.createApolloClient();
            console.log('client loaded!', this.client);
        });
    }
    /**
     * @return {?}
     */
    createApolloClient() {
        return __awaiter(this, void 0, void 0, function* () {
            // TODO: use more shopisticated detection mechanism
            yield delay(500);
            // get client configurations
            /** @type {?} */
            const config = yield this.getClientConfig();
            // HTTP connection to the API
            /** @type {?} */
            const httpLink = createHttpLink({
                uri: config.apiUrl,
                fetch: (/**
                 * @param {?} input
                 * @param {?} init
                 * @return {?}
                 */
                (input, init) => __awaiter(this, void 0, void 0, function* () {
                    /** @type {?} */
                    const conf = yield this.getClientConfig();
                    /** @type {?} */
                    const url = conf.apiUrl || input;
                    // set authorization header
                    if (init.headers && init.headers.set) {
                        init.headers.set('Authorization', `Bearer ${conf.token}`);
                    }
                    else {
                        init.headers = init.headers || ((/** @type {?} */ ({})));
                        init.headers.Authorization = `Bearer ${conf.token}`;
                    }
                    // using eclite offline first fetch
                    // when chrome extension are present
                    if (this.isUsingChromeExtension()) {
                        return window.eclite.fetch(url, init);
                    }
                    return fetch(url, init);
                }))
            });
            // subscriptions link
            /** @type {?} */
            const wsClient = new SubscriptionClient(config.subscriptionUrl, {
                reconnect: true,
                connectionParams: {
                    token: config.token
                }
            }, 
            // using eclite offline first websocket
            // when chrome extension are present
            this.isUsingChromeExtension() ? window.eclite.WebSocket : WebSocket);
            /** @type {?} */
            const wsLink = new WebSocketLink(wsClient);
            // Cache implementation
            /** @type {?} */
            const cache = new InMemoryCache();
            // using the ability to split links, you can send data to each link
            // depending on what kind of operation is being sent
            /** @type {?} */
            const link = split((
            // split based on operation type
            /**
             * @param {?} __0
             * @return {?}
             */
            ({ query }) => {
                /** @type {?} */
                const definition = getMainDefinition(query);
                return (definition.kind === 'OperationDefinition' &&
                    definition.operation === 'subscription');
            }), wsLink, httpLink);
            // Create apollo client
            /** @type {?} */
            const client = new ApolloClient({
                link,
                cache
            });
            return client;
        });
    }
    /**
     * @param {?} config
     * @return {?}
     */
    setClientConfig(config) {
        return this.store.setItem('config', config);
    }
    /**
     * @return {?}
     */
    getClientConfig() {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            const config = yield this.store.getItem('config');
            // using default config when config not present
            if (!config) {
                return {
                    apiUrl: `https://${this.config.ecvcHost}/graphql`,
                    subscriptionUrl: `wss://${this.config.ecvcHost}/subscriptions`,
                    token: null
                };
            }
            return config;
        });
    }
    /**
     * logout and clear client contexts
     * @return {?}
     */
    logout() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.store.removeItem('config');
            window.location.replace('/login');
        });
    }
    /**
     * @param {?} providerId
     * @param {?} password
     * @return {?}
     */
    loginByProvider(providerId, password) {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            const res = yield fetch(`${this.httpProtocol}://${this.config.cloudCentralHost}/login-provider`, {
                method: 'POST',
                mode: 'cors',
                headers: {
                    'Content-Type': 'application/json'
                },
                body: JSON.stringify({
                    provider_id: providerId,
                    password
                })
            });
            if (!res.ok) {
                /** @type {?} */
                let errMessage = 'Gagal melakukan login';
                switch (res.status) {
                    case 404:
                    case 403:
                    case 401:
                        errMessage = 'Kode Faskes & Password tidak cocok';
                        break;
                    case 500:
                        errMessage = 'Maaf, Server kami mengalami gangguan';
                        break;
                }
                throw new Error(errMessage);
            }
            // get token
            const { token } = yield res.json();
            // apply configurations
            yield this.setClientConfig({
                apiUrl: `${this.httpProtocol}://${this.config.ecvcHost}/graphql`,
                subscriptionUrl: `${this.wsProtocol}://${this.config.ecvcHost}/subscriptions`,
                token
            });
        });
    }
    /**
     * @param {?} deviceId
     * @return {?}
     */
    loginByDevice(deviceId) {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            const apiUrl = `${this.httpProtocol}://${deviceId}.${this.config.edgeCN}/graphql`;
            /** @type {?} */
            const subscriptionUrl = `${this.wsProtocol}://${deviceId}.${this.config.edgeCN}/subscriptions`;
            // test connection to edge central
            try {
                yield fetch(apiUrl, {
                    method: 'POST',
                    mode: 'cors',
                    headers: {
                        'Content-Type': 'application/json'
                    },
                    body: JSON.stringify({
                        query: `{
            __schema {
              types {
                name
              }
            }
          }`,
                        operationName: null,
                        variables: {}
                    })
                });
            }
            catch (err) {
                throw new Error('Gagal mendapatkan koneksi, pastikan device anda dalam satu jaringan');
            }
            // apply configurations
            yield this.setClientConfig({
                apiUrl,
                subscriptionUrl,
                token: '--ec-on-device-token--'
            });
        });
    }
}
if (false) {
    /**
     * @type {?}
     * @private
     */
    EdgeCentralClient.prototype.store;
    /** @type {?} */
    EdgeCentralClient.prototype.client;
    /**
     * @type {?}
     * @private
     */
    EdgeCentralClient.prototype.httpProtocol;
    /**
     * @type {?}
     * @private
     */
    EdgeCentralClient.prototype.wsProtocol;
    /**
     * @type {?}
     * @private
     */
    EdgeCentralClient.prototype.config;
}

/**
 * @fileoverview added by tsickle
 * Generated from: lib/ec.module.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Load apollo client on booting
 * @param {?} ec
 * @return {?}
 */
function initEdgeCentralClient(ec) {
    /** @type {?} */
    const loader = (/**
     * @return {?}
     */
    () => ec.loadApolloClient());
    return loader;
}
/**
 * wrapped edge central client so it injectable
 * @param {?} config
 * @return {?}
 */
function provideEdgeCentralService(config) {
    return new EdgeCentralClient(config);
}
/**
 *
 * @export
 */
class EdgeCentralModule {
    /**
     * Creates an instance of EdgeCentralModule.
     * throw error if EdgeCentralModule already created on parent
     * @param {?} parentModule
     */
    constructor(parentModule) {
        if (parentModule) {
            throw new Error('EdgeCentralModule is already loaded. Import it in the AppModule only');
        }
    }
    /**
     * Create new module instance with edge central config service
     *
     * @param {?} config
     * @return {?}
     */
    static forRoot(config) {
        return {
            ngModule: EdgeCentralModule,
            providers: [{ provide: EdgeCentralConfig, useValue: config }]
        };
    }
}
EdgeCentralModule.decorators = [
    { type: NgModule, args: [{
                providers: [
                    {
                        provide: EdgeCentralClient,
                        useFactory: provideEdgeCentralService,
                        deps: [EdgeCentralConfig]
                    },
                    // intiate edge central client on booting
                    {
                        provide: APP_INITIALIZER,
                        useFactory: initEdgeCentralClient,
                        deps: [EdgeCentralClient],
                        multi: true
                    }
                ]
            },] }
];
/** @nocollapse */
EdgeCentralModule.ctorParameters = () => [
    { type: EdgeCentralModule, decorators: [{ type: Optional }, { type: SkipSelf }] }
];

/**
 * @fileoverview added by tsickle
 * Generated from: public-api.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * Generated from: sirusdev-ec-client-angular.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { EdgeCentralClient, EdgeCentralConfig, EdgeCentralModule, initEdgeCentralClient, provideEdgeCentralService };
//# sourceMappingURL=sirusdev-ec-client-angular.js.map
